<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:w="http://niem.gov/niem/wantlist/2"
    xmlns:as="http://www.oasis-open.org/committees/cam"
    xmlns:camed="http://jcam.org.uk/editor">
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, January, 2009 -->
    <!--
        Purpose  : Parse CAM cxf file and create dictionary cross-reference spreadsheet.
                   Inputs: Lookup dictionary of domain element properties definitions.
        Original : 01/01/2009
        Initials : drrw
        Description : Compare exchange template with domain dictionary library in XML lookup format for use in
                      building cross reference spreadsheet.
       
        Updated  - Inits - Change
        01/01/09   drrw   Initial release 
        03/31/09   drrw   Improve matching for namespaced items
        03/31/09   drrw   Check for content type mismatches
        04/28/09   dw,ks  Create NIEM Wantlist using NIEM dictionary and NIEM properties
        05/18/09   drrw   Complete wantlist file output handling
        07/02/09   drrw   Correct issues with NIEM wantlist formatting for SSGT upload
        07/10/09   drrw   Remove need for $results variable and memory issues with wantlist outputting
        07/10/09   drrw   Fix more namespace issues output into wantlist - declare all default NIEM ones
                                    from http://www.niem.gov/niem2/namespaces
        07/10/09   drrw   Rename NIEM-dictionary-properties.xml to NIEM-properties.xml
        07/10/09   drrw   Load NIEM namespaces from file
        07/10/09   drrw   Refactor the wantlist generation to remove unneeded steps, and match correctly.
        07/14/09   drrw   Ensure consistency between wantlist and crossreference name matching algorithms. 
        07/15/09   drrw   Add attributes to wantlist
        07/15/09   drrw   Add fuzzy match lookup section to new spreadsheet for unmatched items
        07/15/09   drrw   Improve performance by optimizing when NIEM wantlist lookups are done
        07/20/09   drrw   Add domain and from into dictionary cross-reference
        07/20/09   drrw   relabel column headings in cross-reference
        08/25/09   drrw   Correct match logic with and without namespaces
        08/25/09   drrw   document() handling with emptyTag
        08/25/09   drrw   Invalid URLresolver and file:/  
        12/07/09   drrw   Add HTML report of comparison results
        12/07/09   drrw   Add logic to matching to use NIEM properties lookup
        12/08/09   drrw   Updates for NIEM 2.1
        12/11/09   drrw   Add links in HTML to files generated
        03/01/10   drrw   Add column to xref XML for item XPath context
        08/18/10   drrw   Refine scoring algorithm on percent NIEM reuse with types
        08/19/10   drrw   Use default NIEM namespace list if external XML file not provided
        08/25/10   drrw   Add logic for default NIEM structure attribute matching
        06/14/11   drrw   Cap percent reuse calculation at 100%
        01/26/12   drrw   Allow match on leaf of names (suggest column)
        01/26/12   drrw   Fix file handles for *NIX platform - with file:// prefix
        04/02/12   drrw   Tweak handling of wantlist generation for correct namespace prefix for types
        03/08/13   SL     Fix file handles for *NIX platform - with file:// prefix
    -->
    <xsl:param name="DictionaryFile">NIEM-dictionary.xml</xsl:param>
    <xsl:param name="resultNamePath">result-xref.xml</xsl:param>

    <xsl:output name="resultXML" method="xml" indent="yes" omit-xml-declaration="yes" />
    <xsl:output name="NIEMresultXML" method="xml" indent="yes" omit-xml-declaration="no" />
    
    <xsl:variable name="version">1.22</xsl:variable>
    <xsl:variable name="annotations">true</xsl:variable>
    <xsl:variable name="ReferenceNames" select="AMOUNT|COUNT|BINARYOBJECT|GRAPHIC|PICTURE|SOUND|VIDEO|CODE|CATEGORY|CURRENCY|EMAIL|DATETIME|DATE|TIME|INDICATOR|FORMAT|LENGTH|WIDTH|HEIGHT|WEIGHT|LEVEL|MEASURE|MODE|METHOD|NUMERIC|NUMBER|PRICE|STATE|STATUS|RANK|FLAG|FREQUENCY|FORMAT|SIZE|UNIT|VALUE|VERSION|RATE|REQUIRED|PERCENT|QUANTITY|QTY|DESCRIPTION|COMMENT|REASON|LOCATION|INSTRUCTIONS|TEXT|TITLE|TYPE|YEAR|MONTH|DAY|NAME|URI|URL|URN|DAYS|HOURS|MINUTES|ID|ID|IDENTIFICATION|IDENTIFIER"/>
    <xsl:variable name="fileURIprefix">file:/</xsl:variable>

    <xsl:variable name="DictionaryFilePath">
        <xsl:choose>
            <xsl:when test="not(contains($DictionaryFile,'\'))"><xsl:value-of select="$DictionaryFile"/></xsl:when><!-- filenames only, no path -->
            <xsl:when test="not(starts-with($DictionaryFile,$fileURIprefix))"><xsl:value-of select="concat($fileURIprefix,replace($DictionaryFile,'\\','/'))"/></xsl:when><!-- path, but no prefix -->
            <xsl:otherwise><xsl:value-of select="replace($DictionaryFile,'\\','/')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="LookupDictionary" select="document($DictionaryFilePath)"/>
    
    
    <xsl:variable name="resultNameFilePath">
        <xsl:choose>
            <xsl:when test="not(contains($resultNamePath,'\'))"><xsl:value-of select="$resultNamePath"/></xsl:when><!-- filenames only, no path -->
            <xsl:when test="not(starts-with($resultNamePath,$fileURIprefix))"><xsl:value-of select="concat($fileURIprefix,replace($resultNamePath,'\\','/'))"/></xsl:when><!-- path, but no prefix -->
            <xsl:otherwise><xsl:value-of select="replace($resultNamePath,'\\','/')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    
    <xsl:variable name="emptyTag"><xsl:element name="empty"/></xsl:variable>
    <xsl:variable name="defaultNIEMns">ansi-nist|apco|atf|can|cap|cbrn|cbrncl|census|de|dea|edxl|edxl-cap|edxl-de|em|fbi|fs|geo|gml|im|intel|ip|it|j|m|nc|om|s|sar|scr|</xsl:variable>
    <xsl:variable name="NIEMstructureAttr">id|metadata|linkMetadata|ref|sequenceID|</xsl:variable>
    
    <xsl:variable name="NIEMnamespaces" select="/as:CAM/as:Namespaces/as:namespace[contains(.,'niem.gov')]/@prefix"/>
    <!-- xsl:variable name="namespaces" select="/as:CAM/as:Namespaces"/ -->
    
    <xsl:variable name="dictionaryPath" select="tokenize($DictionaryFile,'/')"/>
    <xsl:variable name="NIEM-name">
        <xsl:variable name="items" select="count($dictionaryPath)"/>
        <xsl:for-each select="$dictionaryPath"><xsl:if test="position()=$items"><xsl:value-of select="."/></xsl:if></xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="NIEM-PropertyFile">
        <xsl:choose>
            <xsl:when test="contains($DictionaryFile,'NIEM-') and(contains($DictionaryFile,'-dictionary.xml'))"><xsl:value-of select="replace($DictionaryFile,'-dictionary.xml','-properties.xml')"/></xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="NIEM-NamespaceFile" select="replace($NIEM-PropertyFile,'-properties.xml','-namespaces.xml')"/>
    
    <xsl:variable name="CAM-cxf" select="as:CAM/as:AssemblyStructure/as:Structure"/>
    
    <!-- Build filename for unmatched items output file -->
    <xsl:variable name="unmatchedFile">
        <xsl:variable name="filePathStub" select="substring-before($resultNameFilePath,'.xml')"/>
        <xsl:value-of select="if(string-length($filePathStub) &gt; 0) then concat($filePathStub,'-unmatched.xml') else concat($resultNameFilePath,'.unmatched.xml')"/>
    </xsl:variable>
    
    <!-- Read in NIEM Properties if we are processing a NIEM dictionary, otherwise empty. -->
    <xsl:variable name="NIEM-Properties" select="if(string-length($NIEM-PropertyFile) &gt; 0) then document($NIEM-PropertyFile) else $emptyTag"/> 
    <!-- Read in NIEM Namespaces if we are processing a NIEM dictionary, otherwise empty. -->
    <xsl:variable name="defaultNIEMnamespaces" select="if(string-length($NIEM-NamespaceFile) &gt; 0) then document($NIEM-NamespaceFile) else $emptyTag"/>

    <xsl:variable name="inNIEMprefixes" select="if(string-length($NIEM-NamespaceFile) &gt; 0) then $defaultNIEMnamespaces/NIEM//as:namespace/@prefix else tokenize($defaultNIEMnamespaces,'|')"></xsl:variable>
    <xsl:variable name="inNIEMprefixesString"><xsl:for-each select="$inNIEMprefixes"><xsl:value-of select="."/>|</xsl:for-each></xsl:variable>
    <xsl:variable name="root-element" select="$CAM-cxf//as:Element[1]/@name[1]"/>
    
    <xsl:variable name="NIEM-WantlistFile"><!-- Build name of corresponding path to resultNamePath for wantlist output -->
        <xsl:choose>
            <xsl:when test="contains($NIEM-name,'NIEM-')">
                <xsl:choose>
                    <xsl:when test="contains($resultNameFilePath,'/')"><!-- full filename or relative path -->
                        <xsl:choose>
                            <xsl:when test="contains(lower-case($resultNameFilePath),'.xml')">
                                <xsl:variable name="snip-it" select="string-length(substring-before(lower-case($resultNameFilePath),'.xml'))"/>
                                <xsl:value-of select="concat(substring($resultNameFilePath,1,$snip-it),'-wantlist.xml')"/>
                            </xsl:when>
                            <xsl:otherwise><xsl:value-of select="concat($resultNameFilePath,'.wantlist.xml')"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise> <!-- create relative path -->
                        <xsl:choose>
                            <xsl:when test="contains(lower-case($resultNameFilePath),'.xml')">
                                <xsl:variable name="snip-it" select="string-length(substring-before(lower-case($resultNameFilePath),'.xml'))"/>
                                <xsl:value-of select="concat('./',substring($resultNameFilePath,1,$snip-it),'-wantlist.xml')"/>
                            </xsl:when>
                            <xsl:otherwise><xsl:value-of select="concat('./',$resultNameFilePath,'.wantlist.xml')"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
    </xsl:variable>
    
    <!-- Main template, default mode. -->
    
    <xsl:template match="/">
        <xsl:message select="$DictionaryFilePath"/>
        <xsl:result-document exclude-result-prefixes="as camed w" format="resultXML" indent="yes" href="{$resultNameFilePath}">
        <DictionaryXref>
            
            <xsl:comment>
                Cross reference generator, version <xsl:value-of select="$version"/>
                Using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
                - total properties in dictionary - <xsl:value-of select="count($LookupDictionary//Property)"/>
                For Exchange - <xsl:value-of select="if($CAM-cxf/./@ID = $root-element[1]) then $root-element[1] else $CAM-cxf/./@ID"/>
                Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                <xsl:value-of select="'&#xD; &#xD;' "/>
            </xsl:comment>
            <xsl:message>
                Cross reference generator, version <xsl:value-of select="$version"/>
                Using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
                - total properties in dictionary - <xsl:value-of select="count($LookupDictionary//Property)"/>
                For Exchange - <xsl:value-of select="if($CAM-cxf/./@ID = $root-element[1]) then $root-element[1] else $CAM-cxf/./@ID"/>
                Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
            </xsl:message>
            <xsl:message>
                Outputting to <xsl:value-of select="$resultNameFilePath"/>
                Creating cross-reference
            </xsl:message>
            <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/child::*"/>
            <xsl:message>
                Cross-reference finished
            </xsl:message>
        </DictionaryXref>
        </xsl:result-document>
        <!-- xsl:message>[<xsl:copy-of select="$LookupDictionary"/>]</xsl:message -->
        <xsl:message>
            Reporting missing items
        </xsl:message>
        <xsl:variable name="unmatchedItems"><xsl:apply-templates select="/" mode="NOMATCH"/></xsl:variable>
        <xsl:variable name="unmatchedItemsList" select="distinct-values($unmatchedItems/findmatch/@name)"/>
        <xsl:variable name="totalUniqueNodes" select="count(distinct-values($CAM-cxf//as:Element/@name)) + count(distinct-values($CAM-cxf//as:Attribute/@name))"/>
        
        <xsl:result-document exclude-result-prefixes="as camed w" format="resultXML" indent="yes" href="{$unmatchedFile}">         
            
           <xsl:element name="missingItems">
               <xsl:namespace name="w">http://niem.gov/niem/wantlist/2</xsl:namespace>
               <xsl:namespace name="as">http://www.oasis-open.org/committees/cam</xsl:namespace>
               <xsl:namespace name="camed">http://jcam.org.uk/editor</xsl:namespace>
              
               <xsl:comment>
                   Missing items generator, version <xsl:value-of select="$version"/>
                   Using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
                   For Exchange - <xsl:value-of select="if($CAM-cxf/./@ID = $root-element[1]) then $root-element[1] else $CAM-cxf/./@ID"/>
                   Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                   <xsl:value-of select="'&#xD; &#xD;' "/>
               </xsl:comment>
               
                <xsl:for-each select="$unmatchedItemsList">
                    <xsl:variable name="thisItem" select="."/>
                    <xsl:variable name="item" select="$unmatchedItems/findmatch[@name=$thisItem]"/>
                         <xsl:copy-of select="$item[1]"/>
                 </xsl:for-each>
           </xsl:element>
            
        </xsl:result-document>
        <xsl:message>
            Finished missing items 
        </xsl:message>
        
        <xsl:if test="string-length($NIEM-PropertyFile) &gt; 0">
            <xsl:message>
                Generating NIEM wantlist
            </xsl:message>
            <!--  NIEM mode: Load $NIEM-results with in-memory raw results and then write out to wantlist file. -->
            <xsl:variable name="NIEM-results"><xsl:apply-templates select="/" mode="NIEM"/></xsl:variable>
            
            <!-- We have to do this before the result-document step so the variable content can be referenced for statistical contenting step afterwards -->
            <xsl:variable name="uniqueTypesList" select="distinct-values($NIEM-results/w:WantList/w:Type/@w:name)"/>
            <xsl:variable name="NIEM-elementsInType">
                <!-- Process Types which contain ElementInTypes. -->   
                <xsl:for-each select="$uniqueTypesList">
                    <xsl:variable name="thisName" select="."/>
                    <!-- We are pulling all types in types here - regardless of whether the structure actually uses them or not -->
                    <xsl:for-each select="$NIEM-results/w:WantList/w:Type[@w:name=$thisName][1]">
                        <xsl:variable name="findIt" select="@w:name"/>
                        <xsl:variable name="propertyItems" select="$NIEM-Properties//NIEMproperty[@type=$findIt]"></xsl:variable>
                        <xsl:variable name="this-prefix">
                            <xsl:choose>
                                <xsl:when test="$propertyItems[@nsprefix = 'nc']">nc</xsl:when><!-- if NIEM core matched then pick that -->
                                <xsl:otherwise><xsl:value-of select="$propertyItems[1]/@nsprefix"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <!-- xsl:message><xsl:copy-of select="$propertyItems[1]"/></xsl:message -->
                        <xsl:if test="count($propertyItems) &gt; 0">
                            <xsl:element name="w:Type">
                                
                                <xsl:attribute name="w:name"><xsl:value-of select="concat($this-prefix,':',@w:name)"/></xsl:attribute>
                                
                                <xsl:attribute name="w:isRequested" select="'true'" >
                                    <!-- TBD: Kludge: Assume this is always true for now. No NIEM documentation found about this.
                                        We think isRequested=true means user of SSGT tool specifically picked this Type.
                                        isRequested=false means tool determined it is a dependency so is indirectly needed. -->
                                </xsl:attribute>
                                
                                <!-- TBD: We handle nested w:ElementInType elements but not w:AttributeInType or w:Facet at this time. -->
                                <xsl:for-each select="$propertyItems">
                                    <xsl:variable name="w_name" select="@property"/><!-- TBD disabled as not matching anything -->
                                    <xsl:if test="$CAM-cxf//as:Element[@name=$w_name] or($CAM-cxf//as:Element[contains(@name,concat(':',$w_name))])"><!-- Check if the template structure uses this item -->
                                        <xsl:element name="w:ElementInType">
                                            <!-- xsl:attribute name="w:prefix"><xsl:value-of select="@propertyNS"/></xsl:attribute --><!-- NIEM 2.x xsd schema does not use this - so removed -->
                                            <xsl:attribute name="w:name"><xsl:value-of select="concat(@propertyNS,':',@property)"/></xsl:attribute>
                                            <xsl:attribute name="w:isReference"><xsl:value-of select="if (@isReference = '1') then 'true' else 'false'"/></xsl:attribute>
                                        </xsl:element>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:element>
                        </xsl:if>
                    </xsl:for-each> 
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:result-document exclude-result-prefixes="as camed" format="resultXML" indent="yes" href="{$NIEM-WantlistFile}" > 
                
                <!-- NIEM wantlist overall structure handled at this time:
                    w:Wantlist
                    w:Attribute(s)
                    w:Element(s)
                    w:Type(s)
                    w:ElementInType(s)
                -->
                   
                    <xsl:element name="w:WantList" namespace="http://niem.gov/niem/wantlist/2">                    
                    <xsl:attribute name="w:product" select="'NIEM'"></xsl:attribute>
                    <xsl:attribute name="w:release" select="'2.1'"></xsl:attribute>  
                        <xsl:for-each select="$defaultNIEMnamespaces//as:namespace">
                                <xsl:namespace name="{@prefix}" select="."/>
                        </xsl:for-each>
                    <!-- Release number and wantlist/# dependent on NIEM WantList schema version -->
                    
                    <xsl:comment>
                        Cross reference generator, version <xsl:value-of select="$version"/>
                        NIEM Want List generated using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
                        For Exchange - <xsl:value-of select="if($CAM-cxf/./@ID = $root-element[1]) then $root-element[1] else $CAM-cxf/./@ID"/>
                        Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                        <xsl:value-of select="'&#xD; &#xD;' "/>
                        NIEM attributes matched - <xsl:value-of select="count(distinct-values($NIEM-results/w:WantList/w:Attribute/@w:name))"/> 
                        NIEM elements matched - <xsl:value-of select="count(distinct-values($NIEM-results/w:WantList/w:Element/@w:name))"/> 
                        NIEM types matched - <xsl:value-of select="count(distinct-values($NIEM-results/w:WantList/w:Type/@w:name))"/>
                        NIEM sub-items matched - <xsl:value-of select="count($NIEM-elementsInType//w:ElementInType)"/>
                        
                        <xsl:value-of select="'&#xD; &#xD;' "/>
                    </xsl:comment>
                    
                    <!-- xsl:message><xsl:copy-of select="$NIEM-results"/></xsl:message -->
                    
                    <!-- Process Elements separately from Types. --> 
                    <!-- Uniqueness check needed so we don't output 2..N instances of the same w:Name. -->
                    <xsl:variable name="uniqueNamesList" select="distinct-values($NIEM-results/w:WantList/w:Element/@w:name)"/>  
                    <xsl:variable name="uniqueAttributesList" select="distinct-values($NIEM-results/w:WantList/w:Attribute/@w:name)"/> 
                        
                    <!-- Output attributes list -->
                    <xsl:for-each select="$uniqueAttributesList">
                        <xsl:variable name="thisName" select="."/>    
                        <xsl:for-each select="$NIEM-results/w:WantList/w:Attribute[@w:name=$thisName][1]">
                            <xsl:variable name="chkPrefix" select="@w:prefix"/>
                            <!-- xsl:message>attr - <xsl:value-of select="$chkPrefix"/> [<xsl:value-of select="$inNIEMprefixes[.=$chkPrefix]"/>]</xsl:message -->
                            <xsl:if test="$inNIEMprefixes[.=$chkPrefix]">
                                <xsl:element name="w:Attribute">
                                    <!-- xsl:attribute name="w:prefix"><xsl:value-of select="@w:prefix"/></xsl:attribute -->
                                    <xsl:attribute name="w:name"><xsl:value-of select="concat(@w:prefix,':',@w:name)"/></xsl:attribute>
                                </xsl:element>  
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:for-each>
                    
                     <!-- Uniqueness check needed so we don't output 2..N instances of the same w:Type. -->
                     
                    <!-- Output Elements list -->
                    <xsl:for-each select="$uniqueNamesList">
                        <xsl:variable name="thisName" select="."/>    
                        <xsl:for-each select="$NIEM-results/w:WantList/w:Element[@w:name=$thisName][1]">
                            <xsl:variable name="chkPrefix" select="@w:prefix"/>
                            <xsl:if test="$inNIEMprefixes[.=$chkPrefix]">
                                <xsl:element name="w:Element">
                                    <!-- xsl:attribute name="w:prefix"><xsl:value-of select="@w:prefix"/></xsl:attribute -->
                                    <xsl:attribute name="w:name"><xsl:value-of select="concat(@w:prefix,':',@w:name)"/></xsl:attribute>
                                    
                                    <xsl:variable name="findElement" select="@w:name"/>      
                                    <xsl:variable name="refItem" select="$NIEM-Properties//NIEMproperty[@property=$findElement]"></xsl:variable>
                                    <!-- TBD: Not completely sure isReference logic is correct since we can't find NIEM documentation about this. -->
                                    <xsl:attribute name="w:isReference"><xsl:value-of select="if ($refItem/@isReference = '1') then 'true' else 'false'"/></xsl:attribute> 
                                </xsl:element>  
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:for-each>
                    
                    <!-- Output element in type details  -->                    
                        
                   <xsl:copy-of select="$NIEM-elementsInType"/>
                  
                  </xsl:element> <!-- w:Wantlist -->
                  
                <xsl:message>
                    Wantlist finished
                </xsl:message>
            </xsl:result-document>
        
            <!-- Now output diagnostics results -->
            <xsl:variable name="NIEM-attr-count" select="count(distinct-values($NIEM-results/w:WantList/w:Attribute/@w:name))"/>
            <xsl:variable name="NIEM-elmnt-count" select="count(distinct-values($NIEM-results/w:WantList/w:Element/@w:name))"/>
            <xsl:variable name="NIEM-types-count">
                <xsl:variable name="findItemsCount">
                    <xsl:for-each select="$NIEM-elementsInType/w:Type">
                     <xsl:choose>
                         <xsl:when test="count(w:ElementInType) &gt; 0"><xsl:for-each select="w:ElementInType">1|</xsl:for-each></xsl:when>
                         <xsl:otherwise>1|</xsl:otherwise>
                     </xsl:choose>
                 </xsl:for-each>
                </xsl:variable>
                <xsl:value-of select="count(tokenize($findItemsCount,'\|'))"/>
            </xsl:variable>
            <xsl:variable name="NIEM-total" select="$NIEM-attr-count + $NIEM-elmnt-count + $NIEM-types-count"/>
            <xsl:variable name="NIEM-denominator" select="$NIEM-total * 100"/>
            <xsl:variable name="result" select="$NIEM-denominator div($totalUniqueNodes)"/>    
          
            <html>
                <head><title>NIEM dictionary comparison summary</title></head>
                <body>
                    <h1>NIEM Dictionary Comparison Summary</h1>
                    <pre>                        
Cross reference generator, version <xsl:value-of select="$version"/>
NIEM Want List generated using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
For Exchange - <xsl:value-of select="if($CAM-cxf/./@ID = $root-element[1]) then $root-element[1] else concat($CAM-cxf/./@ID,' (', $root-element[1],')')"/>
Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                  </pre>
        <hr/>
                <pre>
                    
        NIEM attributes matched - <xsl:value-of select="$NIEM-attr-count"/> 
        NIEM elements matched - <xsl:value-of select="$NIEM-elmnt-count"/> 
        NIEM types matched - <xsl:value-of select="$NIEM-types-count"/>
           sub-items matched - <xsl:value-of select="count($NIEM-elementsInType//w:ElementInType)"/>
      
        Total NIEM entities matched: <xsl:value-of select="$NIEM-total"/> 
                   
        Unique Exchange Elements and Attributes: <xsl:value-of select="$totalUniqueNodes"/>
           
             </pre>
                    <h2> Percentage NIEM re-use is:  <xsl:value-of select="if(round-half-to-even($result) &gt; 100) then 100 else round-half-to-even($result)"/>%</h2>
            <p/>  
            <hr/>
            <p><a href="{$unmatchedFile}">Unmatched items</a> - <xsl:value-of select="$unmatchedFile"/></p>
            <p><a href="{$resultNameFilePath}">Matched items</a> - <xsl:value-of select="$resultNameFilePath"/></p>
            <p><a href="{$NIEM-WantlistFile}">Wantlist items</a> - <xsl:value-of select="$NIEM-WantlistFile"/></p>
            <hr/>
          </body>
        </html>
        
        </xsl:if> <!-- NIEM -->
        
        <xsl:if test="string-length($NIEM-PropertyFile) &lt; 1"><!-- Not NIEM comparison -->
            
            <html>
                <head><title>Dictionary comparison summary</title></head>
                <body>
                    <h1>Dictionary Comparison Summary</h1>
                    <pre> 
Cross reference generator, version <xsl:value-of select="$version"/>
Generated using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
For Exchange - <xsl:value-of select="if($CAM-cxf/./@ID = $root-element[1]) then $root-element[1] else concat($CAM-cxf/./@ID,' (', $root-element[1],')')"/>
Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                    </pre>
                    <hr/>
                    <pre>

                        Total unmatched items: <xsl:value-of select="count($unmatchedItemsList)"/>
                        
                        Unique Exchange Elements and Attributes: <xsl:value-of select="$totalUniqueNodes"/>
                        
                    </pre>
                    <xsl:variable name="Total-matched" select="$totalUniqueNodes - count($unmatchedItemsList)"/>
                    <xsl:variable name="Match-denominator" select="$Total-matched * 100"/>
                    <xsl:variable name="result" select="$Match-denominator div($totalUniqueNodes)"/>    
                    <h2> Percentage dictionary re-use is:  <xsl:value-of select="if(round-half-to-even($result) &gt; 100) then 100 else round-half-to-even($result)"/>%</h2>
                    <p/>  
                    <hr/>
                    <p><a href="{$unmatchedFile}">Unmatched items</a> - <xsl:value-of select="$unmatchedFile"/></p>
                    <p><a href="{$resultNameFilePath}">Matched items</a> - <xsl:value-of select="$resultNameFilePath"/></p>
                    <hr/>
                </body>
            </html>
        </xsl:if>
        <xsl:message>
            Processing completed
        </xsl:message>
    </xsl:template>
    
    <!-- NIEM Want List processing -->   
    <xsl:template match="/" mode="NIEM">
        
        <w:WantList w:release="2.0" w:product="NIEM" xmlns:w="http://niem.gov/niem/wantlist/2"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://niem.gov Wantlist-niem-1.xsd">               
            <xsl:comment>This collects details into memory variable - for post processing into actual wantlist file.</xsl:comment>
            
            <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/child::*" mode="NIEM"/>
            
        </w:WantList>
    </xsl:template>
    
    <xsl:template match="as:Element">
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <!-- xsl:variable name="nsName" select="if(contains(@name,':')) then substring-after(@name,':') else ''"/>
        <xsl:variable name="nsNameLen" select="string-length($nsName)"/ -->
        <xsl:variable name="itemCount" select="count(//as:Element[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Element[@name=$this][1]/@path"/>
        <xsl:variable name="localPath">
            <xsl:choose>
                <xsl:when test="starts-with(../../parent::*/name(),'as:Element')"><xsl:value-of select="concat('/',../../parent::*/@name,'/',../parent::*/@name,'/',parent::*/@name,'/',$this)"/></xsl:when>
                <xsl:when test="starts-with(../parent::*/name(),'as:Element')"><xsl:value-of select="concat('/',../parent::*/@name,'/',parent::*/@name,'/',$this)"/></xsl:when>
                <xsl:when test="starts-with(parent::*/name(),'as:Element')"><xsl:value-of select="concat('/',parent::*/@name,'/',$this)"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('/',$this)"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:message>this - <xsl:value-of select="$this"/></xsl:message -->
        <xsl:variable name="items">
            <xsl:choose><!-- match on any namespace and name or when name similar  -->
                <xsl:when test="count($LookupDictionary//Property[@item=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[@item=$this][1]"/><!-- xsl:message>match <xsl:value-of select="$this"/></xsl:message --></xsl:when>
                <xsl:when test="count($LookupDictionary//Property[substring-after(@item,':')=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$this][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[substring-after(@item,':')=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$itemName][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[@item=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[@item=$itemName][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)][1]"/></xsl:when>
                <xsl:when test="count($LookupDictionary//Property[upper-case(@item)= upper-case($itemName)]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[upper-case(@item)= upper-case($itemName)][1]"/></xsl:when>
                <xsl:when test="not(contains($ReferenceNames,upper-case($itemName))) and(count($LookupDictionary//Property[ends-with(@item,$itemName)]) &gt; 0)"><!-- Match names leaf and not a reference term -->
                    <xsl:variable name="matchedNames" select="$LookupDictionary//Property[ends-with(@item,$itemName)]/@item"/>
                    <xsl:for-each select="distinct-values($matchedNames)"><xsl:if test="position()=1"><xsl:variable name="this" select="."/><xsl:copy-of select="$LookupDictionary//Property[@item = $this][1]"/></xsl:if></xsl:for-each>
                </xsl:when>
                <xsl:otherwise><!-- If NIEM and no matches then check the Properties dictionary also -->
                    <xsl:if test="string-length($NIEM-PropertyFile) &gt; 0 and(not(contains($ReferenceNames,upper-case($itemName))))">
                        <xsl:variable name="matchedNIEMnames" select="$NIEM-Properties//NIEMproperty[ends-with(@property,$itemName)]/@property"/>
                        <xsl:for-each select="distinct-values($matchedNIEMnames)"><xsl:if test="position()=1"><xsl:variable name="this" select="."/> <xsl:copy-of select="$NIEM-Properties//NIEMproperty[@property = $this][1]"/></xsl:if></xsl:for-each>
                        <!-- xsl:variable name="matchedNIEM" select="$NIEM-Properties//NIEMproperty[ends-with(@property,$itemName)]"/>
                        <xsl:copy-of select="$matchedNIEM[1]"/ -->
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
               
        <xsl:if test="$itemCount=1 or($itemPath[1]=@path) or (count($items) &gt; 0)">
            <xsl:variable name="anno">
                <xsl:choose>
                    <xsl:when test="camed:annotation/camed:documentation[@type='Definition']"><xsl:value-of select="camed:annotation/camed:documentation[@type='Definition']/text()"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="replace(as:Rule/as:constraint/as:annotation/as:documentation[@type='Definition']/text(),'\|','; ')"></xsl:value-of></xsl:otherwise>
                </xsl:choose> 
            </xsl:variable>
            <xsl:variable name="thisMask">
                <xsl:choose>
                    <xsl:when test="$items//@type"><xsl:value-of select="$items//@mask"/></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/></xsl:call-template></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setDateMask')]/@action"/></xsl:call-template></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setNumberMask')]/@action"/></xsl:call-template></xsl:when>
                    <xsl:otherwise><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/></xsl:call-template></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            
            <xsl:variable name="localType">
                <xsl:choose>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'datatype')]"><xsl:value-of select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'datatype')][1]/@action,'('),')')"/>@</xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setType"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="replace(replace($thisMask,' \|',','),'''','')"/></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setDate"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setNumber"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                    <xsl:otherwise>string@<xsl:value-of select="$thisMask"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            
            <xsl:variable name="thisType">
                <xsl:choose>
                    <xsl:when test="$items//@type"><xsl:value-of select="$items//@type"/>@<xsl:value-of select="$thisMask"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$localType"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="typeMatch">
                <xsl:choose>
                    <xsl:when test="count($items//@item) &gt; 0">
                        <xsl:choose>
                            <xsl:when test="$items//@type=''"/>
                            <xsl:when test="$items//@type=substring-before($localType,'@')"/>
                            <xsl:otherwise>of <xsl:value-of select="substring-before($localType,'@')"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise/>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="matchedPrefix" select="$NIEM-Properties//NIEMproperty[ends-with(@property,$itemName)]/@propertyNS"/>
            <xsl:choose>
                <xsl:when test="$annotations='true'" exclude-result-prefixes="#all"><crossref source="{@name}" domain="{substring-before($this,':')}" matched="{$items//@item}" from="{distinct-values($matchedPrefix)}" type="{substring-before($thisType,'@')}" base="{$typeMatch}" definition="{$items//@definition}" annotation="{$anno}" context="{$localPath}"/></xsl:when>
                <xsl:otherwise exclude-result-prefixes="#all"><crossref source="{@name}" domain="{substring-before($this,':')}" matched="{$items//@item}" from="{distinct-values($matchedPrefix)}" type="{substring-before($thisType,'@')}" base="{$typeMatch}" definition="{$items//@definition}" annotation="" context="{$localPath}"/></xsl:otherwise>
           </xsl:choose>
        </xsl:if>
        
        <xsl:apply-templates select="as:Attribute"/>
        <xsl:apply-templates select="as:Element"/>
    </xsl:template>
    
    <xsl:template match="as:Attribute">
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <!-- xsl:variable name="nsName" select="if(contains(@name,':')) then substring-after(@name,':') else ''"></xsl:variable>
        <xsl:variable name="nsNameLen" select="string-length($nsName)"/ -->
        <xsl:variable name="itemCount" select="count(//as:Attribute[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Attribute[@name=$this][1]/@path"/>
        <xsl:variable name="localPath">
            <xsl:choose>
                <xsl:when test="starts-with(../../parent::*/name(),'as:Element')"><xsl:value-of select="concat('/',../../parent::*/@name,'/',../parent::*/@name,'/',parent::*/@name,'/@',$this)"/></xsl:when>
                <xsl:when test="starts-with(../parent::*/name(),'as:Element')"><xsl:value-of select="concat('/',../parent::*/@name,'/',parent::*/@name,'/@',$this)"/></xsl:when>
                <xsl:when test="starts-with(parent::*/name(),'as:Element')"><xsl:value-of select="concat('/',parent::*/@name,'/',$this)"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('/@',$this)"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        
        <xsl:variable name="item">        
                <xsl:choose><!-- match on any namespace and name or when name similar  -->
                    <xsl:when test="count($LookupDictionary//Property[@item=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[@item=$this][1]"/></xsl:when>
                    <xsl:when test="count($LookupDictionary//Property[substring-after(@item,':')=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$this][1]"/></xsl:when>
                    <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[substring-after(@item,':')=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$itemName][1]"/></xsl:when>
                    <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[@item=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[@item=$itemName][1]"/></xsl:when>
                    <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)][1]"/></xsl:when>
                    <xsl:when test="count($LookupDictionary//Property[upper-case(@item)= upper-case($itemName)]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[upper-case(@item)= upper-case($itemName)][1]"/></xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
         
        <xsl:if test="$itemCount=1 or($itemPath[1]=@path)or (count($item) &gt; 0)">
            <!-- xsl:if test="count($item) &gt; 0"><xsl:message>match <xsl:value-of select="$this"/></xsl:message></xsl:if -->
            <xsl:variable name="anno">
                <xsl:choose>
                    <xsl:when test="camed:annotation/camed:documentation[@type='Definition']"><xsl:value-of select="camed:annotation/camed:documentation[@type='Definition']/text()"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="replace(as:Rule/as:constraint/as:annotation/as:documentation[@type='Definition']/text(),'\|','; ')"></xsl:value-of></xsl:otherwise>
                </xsl:choose> 
            </xsl:variable>
            <xsl:variable name="thisMask">
                <xsl:choose>
                    <xsl:when test="$item//@type"><xsl:value-of select="$item//@mask"/></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/></xsl:call-template></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setDateMask')]/@action"/></xsl:call-template></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setNumberMask')]/@action"/></xsl:call-template></xsl:when>
                    <xsl:otherwise><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/></xsl:call-template></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="localType">
                <xsl:choose>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'datatype')]"><xsl:value-of select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'datatype')][1]/@action,'('),')')"/>@</xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setType"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="replace(replace($thisMask,' \|',','),'''','')"/></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setDate"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                    <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setNumber"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                    <xsl:otherwise>string@<xsl:value-of select="$thisMask"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="thisType">
                <xsl:choose>
                    <xsl:when test="$item//@type"><xsl:value-of select="$item//@type"/>@<xsl:value-of select="$thisMask"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$localType"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="typeMatch">
                <xsl:choose>
                    <xsl:when test="count($item//@item) &gt; 0">
                        <xsl:choose>
                            <xsl:when test="$item//@type=''"/>
                            <xsl:when test="$item//@type=substring-before($localType,'@')"/>
                            <xsl:otherwise>of <xsl:value-of select="substring-before($localType,'@')"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise/>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="matchedPrefix" select="$NIEM-Properties//NIEMproperty[@property=$itemName]/@propertyNS"/>
            <xsl:choose>
                <xsl:when test="$annotations='true'" exclude-result-prefixes="#all"><crossref source="{concat('- @',@name)}" domain="{substring-before($this,':')}" matched="{$item//@item}" from="{distinct-values($matchedPrefix)}" type="{substring-before($thisType,'@')}" base="{$typeMatch}" definition="{$item//@definition}" annotation="{$anno}" context="{$localPath}"/></xsl:when>
                <xsl:otherwise exclude-result-prefixes="#all"><crossref source="{concat('- @',@name)}" domain="{substring-before($this,':')}" matched="{$item//@item}" from="{distinct-values($matchedPrefix)}" type="{substring-before($thisType,'@')}" base="{$typeMatch}" definition="{$item//@definition}" annotation="" context="{$localPath}"/></xsl:otherwise>
           </xsl:choose>
       </xsl:if>
    </xsl:template>
    
    <xsl:template match="node()"></xsl:template>
    
    <xsl:template name="setMask">
        <xsl:param name="mask"/>
        <xsl:param name="action"/>
        <xsl:choose>
            <xsl:when test="string-length($mask) &gt; 0"><xsl:value-of select="$mask"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="substring-before(substring-after($action[1],'('),')')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="setDate">
        <xsl:param name="mask"/>
        <xsl:choose>
            <xsl:when test="starts-with($mask,'YYYY-MM-DD''T''')">datetime</xsl:when>
            <xsl:when test="starts-with($mask,'HH:MI')">time</xsl:when>
            <xsl:otherwise>date</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="setNumber">
        <xsl:param name="mask"/>
        <xsl:choose>
            <xsl:when test="contains($mask,'.###')">float</xsl:when>
            <xsl:when test="contains($mask,'.#')">decimal</xsl:when>
            <xsl:when test="$mask='######'">integer</xsl:when>
            <xsl:otherwise>integer</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="setType">
        <xsl:param name="mask"/>
        <xsl:choose>
            <xsl:when test="contains($mask,'Valid'' |''Invalid')">boolean</xsl:when>
            <xsl:when test="contains($mask,'True'' |''False')">boolean</xsl:when>
            <xsl:when test="contains($mask,'true''|''false')">boolean</xsl:when>
            <xsl:when test="contains($mask,'|')">list</xsl:when>
            <xsl:when test="contains($mask,',') and(not(contains($mask,'#')))">list</xsl:when>
            <xsl:otherwise>string</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- For NIEM wantlist, need to separate namespace prefix from element name. -->
    <xsl:template match="as:Element" mode="NIEM" >
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="nsName" select="if(contains(@name,':')) then substring-after(@name,':') else ''"></xsl:variable>
        <xsl:variable name="nsPrefix" select="if(contains(@name,':')) then substring-before(@name,':') else ''"></xsl:variable>
        
        <xsl:variable name="nsNameLen" select="string-length($nsName)"/>
        <xsl:variable name="items">
            <xsl:choose>
                <xsl:when test="count($LookupDictionary//Property[@item=$itemName]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[@item=$itemName][1]"/></xsl:when>                
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        
          <!--  xsl:message><xsl:value-of select="$itemName"/> - <xsl:value-of select="$nsPrefix"/> - <xsl:value-of select="count($items/Property)"/></xsl:message --> 
        
        <xsl:if test="count($items/Property) &gt; 0">
            <xsl:variable name="matchedType" select="$LookupDictionary//Property[@item=$itemName][1]/@type"/>
            <xsl:variable name="matchedPrefix" select="$NIEM-Properties//NIEMproperty[@property=$itemName]/@propertyNS"/>
            <xsl:variable name="typePrefix" select="$NIEM-Properties//NIEMproperty[@type=$matchedType[1]]/@nsprefix"></xsl:variable>
            <xsl:variable name="propPrefix" select="$NIEM-Properties//NIEMproperty[@type=$matchedType[1]]/@propertyNS"></xsl:variable>

           <!-- xsl:message><xsl:value-of select="$itemName"/> - <xsl:value-of select="$typePrefix[1]"/> - <xsl:value-of select="$matchedPrefix[1]"/> - <xsl:value-of select="$matchedType[1]"/></xsl:message --> 

            <xsl:choose>
                 <xsl:when test="count($matchedPrefix) &gt; 0">
                     <xsl:if test="contains($inNIEMprefixesString,concat($nsPrefix,'|'))">                         
                         <!-- Special check for NIEM core types -->
                         <xsl:variable name="domainPrefix">
                             <xsl:choose>
                                 <xsl:when test="$propPrefix[1]='nc'"><!-- check to see if its a generic type - and if so - use item prefix not 'nc' -->
                                     <xsl:choose>
                                         <xsl:when test="$typePrefix[1]='nc' and(count($NIEM-Properties//NIEMproperty[@type=$matchedType[1]]) &gt; 1)">
                                             <xsl:value-of select="if(string-length($nsPrefix) &gt; 0) then $nsPrefix else $matchedPrefix[1]"/>
                                         </xsl:when>
                                         <xsl:otherwise><xsl:value-of select="$matchedPrefix[1]"/></xsl:otherwise>
                                     </xsl:choose>
                                 </xsl:when>
                                 <xsl:otherwise><xsl:value-of select="$matchedPrefix[1]"/></xsl:otherwise>
                             </xsl:choose>
                         </xsl:variable>
                           <w:Element w:prefix="{$domainPrefix}" w:name="{$itemName}"  w:isReference="false"/><!-- reference always false TBD -->   
                     </xsl:if>
                 </xsl:when>
                <xsl:when test="string-length($typePrefix[1]) &gt; 0">
                    <xsl:if test="contains($inNIEMprefixesString,concat($nsPrefix,'|'))">
                        <!-- Special check for NIEM core types -->
                        <xsl:variable name="domainPrefix">
                            <xsl:choose>
                                <xsl:when test="$propPrefix[1]='nc'"><!-- check to see if its a generic type - and if so - use item prefix not 'nc' -->
                                     <xsl:choose>
                                         <xsl:when test="$typePrefix[1]='nc' and(count($NIEM-Properties//NIEMproperty[@type=$matchedType[1]]) &gt; 1)">
                                             <xsl:value-of select="if(string-length($nsPrefix) &gt; 0) then $nsPrefix else $typePrefix[1]"/>
                                         </xsl:when>
                                         <xsl:otherwise><xsl:value-of select="$typePrefix[1]"/></xsl:otherwise>
                                     </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise><xsl:value-of select="$typePrefix[1]"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <w:Element w:prefix="{$domainPrefix}" w:name="{$itemName}"  w:isReference="false"/><!-- reference always false TBD -->   
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
             <xsl:if test="count($typePrefix) &gt; 0">
                 <!-- xsl:message> Type <xsl:value-of select="$this"/>[<xsl:value-of select="$itemName"/>] - <xsl:value-of select="$typePrefix"/>:<xsl:value-of select="$itemName"/></xsl:message -->
                 <w:Type w:prefix="{$typePrefix[1]}" w:name="{$matchedType[1]}" />
             </xsl:if>
        </xsl:if>
        
        <xsl:apply-templates select="as:Attribute" mode="NIEM"/>
        <xsl:apply-templates select="as:Element" mode="NIEM"/>   <!-- Traverse the structure tree -->
    </xsl:template>
    
    <xsl:template match="as:Element" mode="NOMATCH">
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <!-- xsl:variable name="nsName" select="if(contains(@name,':')) then substring-after(@name,':') else ''"></xsl:variable>
        <xsl:variable name="nsNameLen" select="string-length($nsName)"/ -->
        <xsl:variable name="itemCount" select="count(//as:Element[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Element[@name=$this][1]/@path"/>
        <xsl:variable name="items">
            <xsl:choose><!-- match on any namespace and name or when name similar  -->
                <xsl:when test="count($LookupDictionary//Property[@item=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[@item=$this][1]"/></xsl:when>
                <xsl:when test="count($LookupDictionary//Property[substring-after(@item,':')=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$this][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[substring-after(@item,':')=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$itemName][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[@item=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[@item=$itemName][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)][1]"/></xsl:when>
                <xsl:when test="count($LookupDictionary//Property[upper-case(@item)= upper-case($itemName)]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[upper-case(@item)= upper-case($itemName)][1]"/></xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:message> item - <xsl:value-of select="$itemName"/> [<xsl:copy-of select="$items"/>] <xsl:value-of select="count($items/Property)"/></xsl:message -->
        <xsl:choose>
            <xsl:when test="count($items/Property) &gt; 0"/><!-- matched item found --><!-- xsl:message>match <xsl:value-of select="$this"/></xsl:message -->
            <xsl:otherwise><findmatch name="{if(contains(@name,':')) then substring-after(@name,':') else @name}"  domain="{substring-before(@name,':')}" type="element" path="{@path}"/></xsl:otherwise>
        </xsl:choose>
            
        <xsl:apply-templates select="as:Attribute" mode="NOMATCH"/>
        <xsl:apply-templates select="as:Element" mode="NOMATCH"/>   <!-- Traverse the structure tree -->
        
    </xsl:template>
    
    <xsl:template match="as:Attribute" mode="NOMATCH">
        <!-- xsl:message>Check attribute name <xsl:value-of select="@name"/></xsl:message -->
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <!-- xsl:variable name="nsName" select="if(contains(@name,':')) then substring-after(@name,':') else ''"></xsl:variable>
        <xsl:variable name="nsNameLen" select="string-length($nsName)"/ -->
        <xsl:variable name="itemCount" select="count(//as:Element[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Element[@name=$this][1]/@path"/>
        <xsl:variable name="items">
            <xsl:choose><!-- match on any namespace and name or when name similar  -->
                <xsl:when test="count($LookupDictionary//Property[@item=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[@item=$this][1]"/><!-- xsl:message>match <xsl:value-of select="$this"/></xsl:message --></xsl:when>
                <xsl:when test="count($LookupDictionary//Property[substring-after(@item,':')=$this]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$this][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[substring-after(@item,':')=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[substring-after(@item,':')=$itemName][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[@item=$itemName]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[@item=$itemName][1]"/></xsl:when>
                <xsl:when test="contains($this,':') and(count($LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)]) &gt; 0)"><xsl:copy-of select="$LookupDictionary//Property[upper-case(substring-after(@item,':'))= upper-case($itemName)][1]"/></xsl:when>
                <xsl:when test="count($LookupDictionary//Property[upper-case(@item)= upper-case($itemName)]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[upper-case(@item)= upper-case($itemName)][1]"/></xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        
        <xsl:choose>
            <xsl:when test="count($items/Property) &gt; 0"/><!-- matched item found -->
            <xsl:otherwise><findmatch name="{if(contains(@name,':')) then substring-after(@name,':') else @name}"  domain="{substring-before(@name,':')}" type="attribute" path="{@path}"/></xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
    <xsl:template match="as:Attribute" mode="NIEM" >
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="nsName" select="if(contains(@name,':')) then substring-after(@name,':') else ''"></xsl:variable>
        <xsl:variable name="nsPrefix" select="if(contains(@name,':')) then substring-before(@name,':') else ''"></xsl:variable>
        <!-- xsl:message>attribute: <xsl:value-of select="@name"/></xsl:message -->
        <xsl:variable name="nsNameLen" select="string-length($nsName)"/>
        <xsl:variable name="items">
            <xsl:choose>
                <xsl:when test="count($LookupDictionary//Property[@item=$itemName]) &gt; 0"><xsl:copy-of select="$LookupDictionary//Property[@item=$itemName][1]"/></xsl:when>                
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        
        <xsl:if test="count($items/Property) &gt; 0 or(contains($NIEMstructureAttr,concat($itemName,'|')))"><!-- match on Property or if NIEM structure item ??? -->
            <xsl:variable name="matchedType" select="$LookupDictionary//Property[@item=$itemName][1]/@type"/>
            <xsl:variable name="matchedPrefix" select="$NIEM-Properties//NIEMproperty[@property=$itemName]/@propertyNS"/>
            <xsl:variable name="typePrefix" select="$NIEM-Properties//NIEMproperty[@type=$matchedType[1]]/@nsprefix"></xsl:variable>
            <xsl:variable name="propPrefix" select="$NIEM-Properties//NIEMproperty[@type=$matchedType[1]]/@propertyNS"></xsl:variable>
            
            <!-- Special check for NIEM core types -->
            <xsl:variable name="domainPrefix">
                <xsl:choose>
                    <xsl:when test="$propPrefix[1]='nc'"><!-- check to see if its a generic type - and if so - use item prefix not 'nc' -->
                        <xsl:choose>
                            <xsl:when test="$typePrefix[1]='nc' and(count($NIEM-Properties//NIEMproperty[@type=$matchedType[1]]) &gt; 1)">
                                <xsl:value-of select="if(string-length($nsPrefix) &gt; 0) then $nsPrefix else $typePrefix[1]"/>
                            </xsl:when>
                            <xsl:otherwise><xsl:value-of select="$typePrefix[1]"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="contains($NIEMstructureAttr,concat($itemName,'|'))">s</xsl:when><!-- default NIEM structure attribute -->
                    <xsl:otherwise><xsl:value-of select="$typePrefix[1]"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            
            <!-- xsl:message><xsl:value-of select="$itemName"/> - <xsl:value-of select="$domainPrefix"/> < xsl:value-of select="$typePrefix[1]"/> - <xsl:value-of select="$matchedPrefix[1]"/> - <xsl:value-of select="$matchedType[1]"/></xsl:message -->
           
           <w:Attribute w:prefix="{$domainPrefix}" w:name="{$itemName}"  />

        </xsl:if>
        <!-- No match so skip from wantlist output -->      
        
    </xsl:template>
    
</xsl:stylesheet>
